@@ -15,12 +15,26 @@
 """
 import unittest
 
+# pylint:disable=inherit-non-class,no-method-argument,no-self-argument
 
 class Test_verifyClass(unittest.TestCase):
 
-    def _callFUT(self, iface, klass):
+    verifier = None
+
+    def setUp(self):
+        self.verifier = self._get_FUT()
+
+    @classmethod
+    def _get_FUT(cls):
         from zope.interface.verify import verifyClass
-        return verifyClass(iface, klass)
+        return verifyClass
+
+    _adjust_object_before_verify = lambda self, x: x
+
+    def _callFUT(self, iface, klass, **kwargs):
+        return self.verifier(iface,
+                             self._adjust_object_before_verify(klass),
+                             **kwargs)
 
     def test_class_doesnt_implement(self):
         from zope.interface import Interface
@@ -54,7 +68,8 @@ class Test_verifyClass(unittest.TestCase):
         from zope.interface.exceptions import BrokenImplementation
 
         class ICurrent(Interface):
-            def method(): pass
+            def method():
+                """docstring"""
 
         @implementer(ICurrent)
         class Current(object):
@@ -68,7 +83,8 @@ class Test_verifyClass(unittest.TestCase):
         from zope.interface import implementer
 
         class ICurrent(Interface):
-            def method(): pass
+            def method():
+                """docstring"""
 
         @implementer(ICurrent)
         class Current(object):
@@ -85,7 +101,7 @@ class Test_verifyClass(unittest.TestCase):
 
         class IBase(Interface):
             def method():
-                pass
+                """docstring"""
 
         class IDerived(IBase):
             pass
@@ -103,7 +119,7 @@ class Test_verifyClass(unittest.TestCase):
 
         class IBase(Interface):
             def method():
-                pass
+                """docstring"""
 
         class IDerived(IBase):
             pass
@@ -124,7 +140,7 @@ class Test_verifyClass(unittest.TestCase):
         class ICurrent(Interface):
 
             def method(a):
-                pass
+                """docstring"""
 
         @implementer(ICurrent)
         class Current(object):
@@ -142,7 +158,7 @@ class Test_verifyClass(unittest.TestCase):
         class ICurrent(Interface):
 
             def method(a):
-                pass
+                """docstring"""
 
         @implementer(ICurrent)
         class Current(object):
@@ -161,7 +177,7 @@ class Test_verifyClass(unittest.TestCase):
         class ICurrent(Interface):
 
             def method(*args):
-                pass
+                """docstring"""
 
         @implementer(ICurrent)
         class Current(object):
@@ -180,7 +196,7 @@ class Test_verifyClass(unittest.TestCase):
         class ICurrent(Interface):
 
             def method(**kw):
-                pass
+                """docstring"""
 
         @implementer(ICurrent)
         class Current(object):
@@ -199,7 +215,7 @@ class Test_verifyClass(unittest.TestCase):
         class ICurrent(Interface):
 
             def method(a):
-                pass
+                """docstring"""
 
         @implementer(ICurrent)
         class Current(object):
@@ -217,7 +233,7 @@ class Test_verifyClass(unittest.TestCase):
         class ICurrent(Interface):
 
             def method(a):
-                pass
+                """docstring"""
 
         @implementer(ICurrent)
         class Current(object):
@@ -234,7 +250,7 @@ class Test_verifyClass(unittest.TestCase):
         class ICurrent(Interface):
 
             def method(a):
-                pass
+                """docstring"""
 
         @implementer(ICurrent)
         class Current(object):
@@ -252,7 +268,7 @@ class Test_verifyClass(unittest.TestCase):
         class ICurrent(Interface):
 
             def method(a):
-                pass
+                """docstring"""
 
         @implementer(ICurrent)
         class Current(object):
@@ -270,7 +286,7 @@ class Test_verifyClass(unittest.TestCase):
         class ICurrent(Interface):
 
             def method(a):
-                pass
+                """docstring"""
 
         @implementer(ICurrent)
         class Current(object):
@@ -287,7 +303,7 @@ class Test_verifyClass(unittest.TestCase):
         class ICurrent(Interface):
 
             def method(a):
-                pass
+                """docstring"""
 
         @implementer(ICurrent)
         class Current(object):
@@ -305,7 +321,7 @@ class Test_verifyClass(unittest.TestCase):
         class ICurrent(Interface):
 
             def method(a, *args):
-                pass
+                """docstring"""
 
         @implementer(ICurrent)
         class Current(object):
@@ -323,7 +339,7 @@ class Test_verifyClass(unittest.TestCase):
         class ICurrent(Interface):
 
             def method(a, *args):
-                pass
+                """docstring"""
 
         @implementer(ICurrent)
         class Current(object):
@@ -340,7 +356,7 @@ class Test_verifyClass(unittest.TestCase):
         class ICurrent(Interface):
 
             def method(a, *args):
-                pass
+                """docstring"""
 
         @implementer(ICurrent)
         class Current(object):
@@ -357,7 +373,7 @@ class Test_verifyClass(unittest.TestCase):
         class ICurrent(Interface):
 
             def method(**kwargs):
-                pass
+                """docstring"""
 
         @implementer(ICurrent)
         class Current(object):
@@ -375,7 +391,7 @@ class Test_verifyClass(unittest.TestCase):
         class ICurrent(Interface):
 
             def method(*args):
-                pass
+                """docstring"""
 
         @implementer(ICurrent)
         class Current(object):
@@ -395,7 +411,7 @@ class Test_verifyClass(unittest.TestCase):
         class ICurrent(Interface):
 
             def method(**kwargs):
-                pass
+                """docstring"""
 
         @implementer(ICurrent)
         class Current(object):
@@ -430,7 +446,7 @@ class Test_verifyClass(unittest.TestCase):
 
         class ICurrent(Interface):
             def method():
-                pass
+                """docstring"""
 
         @implementer(ICurrent)
         class Current:
@@ -503,7 +519,7 @@ class Test_verifyClass(unittest.TestCase):
         class ICurrent(Interface):
 
             def method(a):
-                pass
+                """docstring"""
 
         @implementer(ICurrent)
         class Current(object):
@@ -514,13 +530,71 @@ class Test_verifyClass(unittest.TestCase):
 
         self._callFUT(ICurrent, Current)
 
+    def test_dict_IFullMapping(self):
+        # A dict should be an IFullMapping, but this exposes two
+        # issues. First, on CPython, methods of builtin types are
+        # "method_descriptor" objects, and are harder to introspect.
+        # Second, on PyPy, the signatures can be just plain wrong,
+        # specifying as required arguments that are actually optional.
+        # See https://github.com/zopefoundation/zope.interface/issues/118
+        from zope.interface.common.mapping import IFullMapping
+        self._callFUT(IFullMapping, dict, tentative=True)
+
+    def test_list_ISequence(self):
+        # As for test_dict_IFullMapping
+        from zope.interface.common.sequence import ISequence
+        self._callFUT(ISequence, list, tentative=True)
+
+    def test_tuple_IReadSequence(self):
+        # As for test_dict_IFullMapping
+        from zope.interface.common.sequence import IReadSequence
+        self._callFUT(IReadSequence, tuple, tentative=True)
+
+
+    def test_multiple_invalid(self):
+        from zope.interface.exceptions import MultipleInvalid
+        from zope.interface.exceptions import DoesNotImplement
+        from zope.interface.exceptions import BrokenImplementation
+        from zope.interface import Interface
+        from zope.interface import classImplements
+
+        class ISeveralMethods(Interface):
+            def meth1(arg1):
+                "Method 1"
+            def meth2(arg1):
+                "Method 2"
+
+        class SeveralMethods(object):
+            pass
+
+        with self.assertRaises(MultipleInvalid) as exc:
+            self._callFUT(ISeveralMethods, SeveralMethods)
+
+        ex = exc.exception
+        self.assertEqual(3, len(ex.exceptions))
+        self.assertIsInstance(ex.exceptions[0], DoesNotImplement)
+        self.assertIsInstance(ex.exceptions[1], BrokenImplementation)
+        self.assertIsInstance(ex.exceptions[2], BrokenImplementation)
+
+        # If everything else is correct, only the single error is raised without
+        # the wrapper.
+        classImplements(SeveralMethods, ISeveralMethods)
+        SeveralMethods.meth1 = lambda self, arg1: "Hi"
+
+        with self.assertRaises(BrokenImplementation):
+            self._callFUT(ISeveralMethods, SeveralMethods)
+
 class Test_verifyObject(Test_verifyClass):
 
-    def _callFUT(self, iface, target):
+    @classmethod
+    def _get_FUT(cls):
         from zope.interface.verify import verifyObject
+        return verifyObject
+
+    def _adjust_object_before_verify(self, target):
         if isinstance(target, (type, type(OldSkool))):
             target = target()
-        return verifyObject(iface, target)
+        return target
 
     def test_class_misses_attribute_for_attribute(self):
         # This check *fails* for verifyObject
